<?php

namespace Hyiplab\Controllers\Admin;

use Hyiplab\BackOffice\Request;
use Hyiplab\Controllers\Controller;
use Hyiplab\Models\Deposit;
use Hyiplab\Models\SupportTicket;
use Hyiplab\Models\Transaction;
use Hyiplab\Models\User;
use Hyiplab\Models\Withdrawal;

class UserController extends Controller
{
    public function allUsers()
    {
        $request   = new Request();
        $pageTitle = "All Users";
        if ($request->search) {
            $users = User::where('user_login', urldecode($request->search))->orWhere('user_email', '=', urldecode($request->search))->orderBy('id', 'DESC')->paginate(hyiplab_paginate());
        } else {
            $users = User::orderBy('id', 'DESC')->paginate(hyiplab_paginate());
        }
        $this->view('admin/users/list', compact('pageTitle', 'users'));
    }

    public function userDetail()
    {
        $request = new Request();
        $user    = User::find($request->id);
        if (!$user) {
            hyiplab_abort(404);
        }
        $pageTitle        = "User Detail - " . $user->user_login;
        $countries        = json_decode(file_get_contents(HYIPLAB_ROOT . 'views/partials/country.json'));
        $totalDeposit     = Deposit::where('user_id', $user->ID)->where('status', 1)->sum('amount');
        $totalWithdrawals = Withdrawal::where('user_id', $user->ID)->where('status', 1)->sum('amount');
        $totalTransaction = Transaction::where('user_id', $user->ID)->count();
        $pendingTicket    = SupportTicket::whereIn('status', [0, 2])->where('user_id', $user->ID)->count();
        $this->view('admin/users/detail', compact('pageTitle', 'user', 'countries', 'totalDeposit', 'totalWithdrawals', 'totalTransaction', 'pendingTicket'));
    }

    public function userUpdate()
    {
        $request = new Request();
        $request->validate([
            'display_name' => 'required',
            'email'        => 'required|email',
            'mobile'       => 'required',
            'country'      => 'required'
        ]);
        $user = User::find($request->id);
        if (!$user) {
            hyiplab_abort(404);
        }

        $countryData  = json_decode(file_get_contents(HYIPLAB_ROOT . 'views/partials/country.json'));
        $countryCode  = $request->country;
        $country      = $countryData->$countryCode->country;
        $dialCode     = $countryData->$countryCode->dial_code;

        $userData = [
            'ID'           => intval($user->ID),
            'display_name' => sanitize_text_field($request->display_name)
        ];

        $user_id = wp_update_user($userData);

        update_user_meta($user_id, 'hyiplab_mobile', sanitize_text_field($dialCode . $request->mobile));
        update_user_meta($user_id, 'hyiplab_country_code', sanitize_text_field($countryCode));
        update_user_meta($user_id, 'hyiplab_country', sanitize_text_field($country));
        update_user_meta($user_id, 'hyiplab_address', sanitize_text_field($request->address));
        update_user_meta($user_id, 'hyiplab_city', sanitize_text_field($request->city));
        update_user_meta($user_id, 'hyiplab_state', sanitize_text_field($request->state));
        update_user_meta($user_id, 'hyiplab_zip', sanitize_text_field($request->zip));

        update_user_meta($user->ID, 'hyiplab_user_active', $request->user_active ? 1 : 0);

        if ($request->email_verify) {
            delete_user_meta($user->ID, 'hyiplab_email_verify');
        } else {
            if (!get_user_meta($user->ID, 'hyiplab_email_verify', true)) {
                $verify_email = wp_generate_password(20, false);
                update_user_meta($user->ID, 'hyiplab_email_verify', $verify_email);
                $verify_link = sprintf('%s?email=verify&login=%s&key=%s', hyiplab_route_link('user.login'), rawurlencode($user->user_login), $verify_email);
                hyiplab_notify($user, 'REGISTER', [
                    'verify_link' => esc_url($verify_link)
                ]);
            }
        }

        $notify[] = ['success', 'User details updated successfully'];
        hyiplab_back($notify);
    }
}
