<?php

namespace Hyiplab\Controllers\User;

use Hyiplab\BackOffice\Request;
use Hyiplab\Controllers\Controller;
use Hyiplab\Models\Deposit;
use Hyiplab\Models\Invest;
use Hyiplab\Models\Transaction;
use Hyiplab\Models\User;
use Hyiplab\Models\Withdrawal;

class DashboardController extends Controller
{

    public function index()
    {
        global $user_ID, $wpdb;

        $this->pageTitle = 'Dashboard';

        $user         = get_userdata($user_ID);
        $data['user'] = $user;

        $data['submittedDeposits']  = Deposit::where('status', '!=', 0)->where('user_id', $user->ID)->sum('amount');
        $data['successfulDeposits'] = Deposit::where('status', 1)->where('user_id', $user->ID)->sum('amount');
        $data['requestedDeposits']  = Deposit::where('user_id', $user->ID)->sum('amount');
        $data['initiatedDeposits']  = Deposit::where('status', 0)->where('user_id', $user->ID)->sum('amount');
        $data['pendingDeposits']    = Deposit::where('status', 2)->where('user_id', $user->ID)->sum('amount');
        $data['rejectedDeposits']   = Deposit::where('status', 3)->where('user_id', $user->ID)->sum('amount');

        $data['submittedWithdraws']  = Withdrawal::where('status', '!=', 0)->where('user_id', $user->ID)->sum('amount');
        $data['successfulWithdraws'] = Withdrawal::where('status', 1)->where('user_id', $user->ID)->sum('amount');
        $data['requestedWithdraws']  = Withdrawal::where('user_id', $user->ID)->sum('amount');
        $data['initiatedWithdraws']  = Withdrawal::where('status', 0)->where('user_id', $user->ID)->sum('amount');
        $data['pendingWithdraws']    = Withdrawal::where('status', 2)->where('user_id', $user->ID)->sum('amount');
        $data['rejectedWithdraws']   = Withdrawal::where('status', 3)->where('user_id', $user->ID)->sum('amount');

        $data['invests']               = Invest::where('user_id', $user->ID)->sum('amount');
        $data['completedInvests']      = Invest::where('user_id', $user->ID)->where('status', 0)->sum('amount');
        $data['runningInvests']        = Invest::where('user_id', $user->ID)->where('status', 1)->sum('amount');
        $data['interests']             = Transaction::where('remark', 'interest')->where('user_id', $user->ID)->sum('amount');
        $data['depositWalletInvests']  = Invest::where('user_id', $user->ID)->where('wallet_type', 'deposit_wallet')->where('status', 1)->sum('amount');
        $data['interestWalletInvests'] = Invest::where('user_id', $user->ID)->where('wallet_type', 'interest_wallet')->where('status', 1)->sum('amount');

        $table_prefix  = $wpdb->base_prefix;
        $date          = hyiplab_date()->subDays(30)->toDate();
        $data['chart'] = Transaction::selectRaw("select SUM(amount) as amount, DATE_FORMAT(created_at,'%Y-%m-%d') as date from " . $table_prefix . "hyiplab_transactions where `remark` = 'interest' and `created_at` >= '" . $date . "' and `user_id` = " . $user->ID . " group by `date` order by `created_at` asc");
        return $this->view('user/dashboard', compact('data'));
    }

    public function inactive()
    {
        global $user_ID;
        $this->pageTitle = 'Inactive';
        if (get_user_meta($user_ID, 'hyiplab_user_active', true) != 0) {
            hyiplab_redirect(hyiplab_route_link('user.home'));
        }
        return $this->view('user/inactive');
    }

    public function transferBalance()
    {
        global $user_ID;
        if (!get_option('hyiplab_balance_transfer')) {
            hyiplab_abort(404);
        }
        $this->pageTitle = "Balance Transfer";
        $user = get_userdata($user_ID);
        $this->view('user/balance_transfer', compact('user'));
    }

    public function transferBalanceSubmit()
    {
        global $user_ID;
        if (!get_option('hyiplab_balance_transfer')) {
            hyiplab_abort(404);
        }

        $request = new Request();
        $request->validate([
            'username' => 'required',
            'amount'   => 'required|numeric',
            'wallet'   => 'required|in:deposit_wallet,interest_wallet',
        ]);

        $user = get_userdata($user_ID);

        if ($user->user_login == $request->username) {
            $notify[] = ['error', 'You cannot transfer balance to your own account'];
            hyiplab_back($notify);
        }

        $receiver = User::where('user_login', $request->username)->first();
        if (!$receiver) {
            $notify[] = ['error', 'Oops! Receiver not found'];
            hyiplab_back($notify);
        }

        $charge      = get_option('hyiplab_balance_transfer_fixed_charge') + ($request->amount * get_option('hyiplab_balance_transfer_percent_charge')) / 100;
        $afterCharge = $request->amount + $charge;
        $wallet      = $request->wallet;

        $userBalance = hyiplab_balance($user->ID, $wallet);

        if ($userBalance < $afterCharge) {
            $notify[] = ['error', 'You have no sufficient balance to this wallet'];
            hyiplab_back($notify);
        }

        $userAfterBalance = $userBalance - $afterCharge;
        update_user_meta($user->ID, 'hyiplab_'.$wallet, $userAfterBalance);

        $transaction               = new Transaction();
        $transaction->user_id      = $user->ID;
        $transaction->amount       = hyiplab_get_amount($afterCharge);
        $transaction->charge       = $charge;
        $transaction->trx_type     = '-';
        $transaction->trx          = hyiplab_trx();
        $transaction->wallet_type  = $wallet;
        $transaction->remark       = 'balance_transfer';
        $transaction->details      = 'Balance transfer to ' . $receiver->user_login;
        $transaction->post_balance = hyiplab_get_amount($userAfterBalance);
        $transaction->created_at   = current_time('mysql');
        $transaction->save();

        $receiverBalance = hyiplab_balance($receiver->ID, 'deposit_wallet');
        $receiverAfterBalance = $userBalance + $request->amount;
        update_user_meta($receiver->ID, 'hyiplab_deposit_wallet', $receiverAfterBalance);

        $transaction               = new Transaction();
        $transaction->user_id      = $receiver->ID;
        $transaction->amount       = hyiplab_get_amount($request->amount);
        $transaction->charge       = 0;
        $transaction->trx_type     = '+';
        $transaction->trx          = hyiplab_trx();
        $transaction->wallet_type  = 'deposit_wallet';
        $transaction->remark       = 'balance_transfer';
        $transaction->details      = 'Balance received from ' . $user->user_login;
        $transaction->post_balance = hyiplab_get_amount($receiverAfterBalance);
        $transaction->created_at   = current_time('mysql');
        $transaction->save();

        $notify[] = ['success', 'Balance transferred successfully'];
        hyiplab_back($notify);
    }
}
