<?php

namespace Hyiplab\Hook;

use Hyiplab\Lib\HyipLab;
use Hyiplab\Models\Invest;
use Hyiplab\Models\Plan;
use Hyiplab\Models\Transaction;
use Hyiplab\Models\User;
use Hyiplab\Models\UserRanking;

class Cron
{
    public function cron()
    {
        $now = current_time('mysql');

        update_option('hyiplab_last_cron_run', $now);

        $invests = Invest::where('status', 1)->where('next_time', '<=', $now)->orderBy('last_time')->limit(100)->get();
        foreach ($invests as $invest) {
            $updateInvest                    = [];
            $updateInvest['return_rec_time'] = $invest->return_rec_time + 1;
            $updateInvest['paid']            = $invest->paid + $invest->interest;
            $updateInvest['should_pay']      = $invest->should_pay - ($invest->period > 0 ? $invest->interest : 0);
            $updateInvest['next_time']       = hyiplab_date()->addHours($invest->hours)->toDateTime();
            $updateInvest['last_time']       = current_time('mysql');

            $afterBalance = hyiplab_balance($invest->user_id, 'interest_wallet') + $invest->interest;
            update_user_meta($invest->user_id, "hyiplab_interest_wallet", $afterBalance);

            $plan = Plan::where('id', $invest->plan_id)->first();
            if (!$plan) {
                continue;
            }
            $user = User::where('ID', $invest->user_id)->first();
            if (!$user) {
                continue;
            }
            $trx          = hyiplab_trx();
            $transactions = [
                'user_id'      => $invest->user_id,
                'invest_id'    => $invest->id,
                'amount'       => $invest->interest,
                'post_balance' => $afterBalance,
                'charge'       => 0,
                'trx_type'     => '+',
                'details'      => hyiplab_show_amount($invest->interest) . ' ' . hyiplab_currency('text') . esc_html__(' interest from ', HYIPLAB_PLUGIN_NAME) . esc_html($plan->name),
                'trx'          => $trx,
                'wallet_type'  => 'interest_wallet',
                'remark'       => 'interest',
                'created_at'   => current_time('mysql')
            ];
            Transaction::insert($transactions);

            if (get_option('hyiplab_invest_return_commission') == 1) {
                $commissionType = 'invest_return_commission';
                HyipLab::levelCommission($user, $invest->interest, $commissionType, $trx);
            }

            if ($invest->return_rec_time >= $invest->period && $invest->period != -1) {
                $updateInvest['status'] = 0;

                if ($invest->capital_status == 1) {
                    $capital      = $invest->amount;
                    $afterBalance = hyiplab_balance($invest->user_id, 'interest_wallet') + $capital;
                    update_user_meta($invest->user_id, "hyiplab_interest_wallet", $afterBalance);

                    $transactions = [
                        'user_id'      => $invest->user_id,
                        'invest_id'    => $invest->id,
                        'amount'       => $capital,
                        'post_balance' => $afterBalance,
                        'charge'       => 0,
                        'trx_type'     => '+',
                        'details'      => hyiplab_show_amount($capital) . ' ' . hyiplab_currency('text') . esc_html__('capital back from', HYIPLAB_PLUGIN_NAME) . esc_html($plan->name),
                        'trx'          => $trx,
                        'wallet_type'  => 'interest_wallet',
                        'remark'       => 'interest',
                        'created_at'   => current_time('mysql')
                    ];

                    Transaction::insert($transactions);
                }
            }

            Invest::where('id', $invest->id)->update($updateInvest);

            $plan = get_hyiplab_plan($invest->plan_id);

            hyiplab_notify($user, 'INTEREST', [
                'trx'          => $trx,
                'amount'       => hyiplab_show_amount($invest->interest),
                'plan_name'    => esc_html($plan->name),
                'post_balance' => hyiplab_show_amount($afterBalance),
            ]);
        }
    }

    public function rank()
    {
        if (!get_option('hyiplab_user_ranking')) {
            return 'MODULE DISABLED';
        }

        $args = array(
            'meta_query' => array(
                'relation' => 'OR',
                array(
                    'key' => 'hyiplab_last_rank_update',
                    'compare' => 'EXISTS',
                ),
                array(
                    'key' => 'hyiplab_last_rank_update',
                    'compare' => 'NOT EXISTS',
                ),
            ),
            'orderby'  => 'meta_value',
            'order'    => 'desc',
            'number' => 50
        );
        $user_query = new \WP_User_Query($args);
        $users = $user_query->get_results();

        foreach ($users as $user) {

            update_user_meta($user->ID, 'hyiplab_last_rank_update', current_time('timestamp'));
            $userInvests = get_user_meta($user->ID, 'hyiplab_total_invest', true);
            $referralInvests = get_user_meta($user->ID, 'hyiplab_team_invest', true);
            $referralCount = getViserAllReferrer($user->ID, true);
            $userRankingId = get_user_meta($user->ID, 'hyiplab_ranking_id', true);

            $rankings = UserRanking::where('status', 1)->where('id', '>', $userRankingId)->where('minimum_invest', '<=', $userInvests)->where('min_referral_invest', '<=', $referralInvests)->where('min_referral', '<=', $referralCount)->get();

            foreach ($rankings as $ranking) {

                $afterBalance = hyiplab_balance($user->ID, 'interest_wallet') + $ranking->bonus;
                update_user_meta($user->ID, "hyiplab_interest_wallet", $afterBalance);
                update_user_meta($user->ID, "hyiplab_ranking_id", $ranking->id);

                $transaction               = new Transaction();
                $transaction->user_id      = $user->ID;
                $transaction->amount       = $ranking->bonus;
                $transaction->charge       = 0;
                $transaction->post_balance = $afterBalance;
                $transaction->trx_type     = '+';
                $transaction->trx          = hyiplab_trx();
                $transaction->remark       = 'ranking_bonus';
                $transaction->wallet_type  = 'interest_wallet';
                $transaction->details      = hyiplab_show_amount($ranking->bonus) . ' ' . hyiplab_currency('text') . ' ranking bonus for ' . $ranking->name;
                $transaction->created_at  = current_time('mysql');
                $transaction->save();
            }
        }
    }
}
